@@ -1,6 +1,7 @@
 /*
 	Audio File Library
 	Copyright (C) 1998-2000, Michael Pruett <michael@68k.org>
+	Copyright (C) 2000, Silicon Graphics, Inc.
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Library General Public
@@ -13,8 +14,8 @@
 	Library General Public License for more details.
 
 	You should have received a copy of the GNU Library General Public
-	License along with this library; if not, write to the 
-	Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
+	License along with this library; if not, write to the
+	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 	Boston, MA  02111-1307  USA.
 */
 
@@ -40,6 +41,9 @@
 #include "afinternal.h"
 #include "byteorder.h"
 #include "aiff.h"
+#include "setup.h"
+#include "track.h"
+#include "marker.h"
 
 static status ParseFVER (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 	size_t size);
@@ -75,6 +79,21 @@ int _af_aiffc_compression_types[_AF_AIFF_NUM_COMPTYPES] =
 	AF_COMPRESSION_G711_ALAW
 };
 
+_AFfilesetup _af_aiff_default_filesetup =
+{
+	_AF_VALID_FILESETUP,	/* valid */
+	AF_FILE_AIFF,		/* fileFormat */
+	AF_TRUE,		/* trackSet */
+	AF_TRUE,		/* instrumentSet */
+	AF_TRUE,		/* miscellaneousSet */
+	1,			/* trackCount */
+	NULL,			/* tracks */
+	1,			/* instrumentCount */
+	NULL,			/* instruments */
+	0,			/* miscellaneousCount */
+	NULL			/* miscellaneous */
+};
+
 /*
 	FVER chunks are only present in AIFF-C files.
 */
@@ -96,14 +115,24 @@ static status ParseFVER (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 */
 static status ParseAESD (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type, size_t size)
 {
+	_Track		*track;
 	unsigned char	aesChannelStatusData[24];
 
 	assert(!memcmp(&type, "AESD", 4));
 	assert(size == 24);
 
-	file->aesDataPresent = 1;
-	af_fread(aesChannelStatusData, 1, 24, fh);
-	memcpy(file->aesData, aesChannelStatusData, 24);
+	track = _af_filehandle_get_track(file, AF_DEFAULT_TRACK);
+
+	track->hasAESData = AF_TRUE;
+
+	/*
+		Try to read 24 bytes of AES nonaudio data from the file.
+		Fail if the file disappoints.
+	*/
+	if (af_fread(aesChannelStatusData, 1, 24, fh) != 24)
+		return AF_FAIL;
+
+	memcpy(track->aesData, aesChannelStatusData, 24);
 
 	return AF_SUCCEED;
 }
@@ -115,25 +144,16 @@ static status ParseAESD (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 static status ParseMiscellaneous (AFfilehandle file, AF_VirtualFile *fh,
 	u_int32_t type, size_t size)
 {
-	int	misctype = 0;
+	int	misctype = AF_MISC_UNRECOGNIZED;
 
 	assert(!memcmp(&type, "NAME", 4) || !memcmp(&type, "AUTH", 4) ||
 		!memcmp(&type, "(c) ", 4) || !memcmp(&type, "ANNO", 4) ||
 		!memcmp(&type, "APPL", 4) || !memcmp(&type, "MIDI", 4));
 	assert(size >= 0);
 
-	if (file->miscellaneousCount == 0)
-	{
-		assert(file->miscellaneous == NULL);
-		file->miscellaneousCount++;
-		file->miscellaneous = _af_malloc(sizeof (struct _Miscellaneous));
-	}
-	else
-	{
-		file->miscellaneousCount++;
-		file->miscellaneous = _af_realloc(file->miscellaneous,
-			file->miscellaneousCount * sizeof (struct _Miscellaneous));
-	}
+	file->miscellaneousCount++;
+	file->miscellaneous = _af_realloc(file->miscellaneous,
+		file->miscellaneousCount * sizeof (_Miscellaneous));
 
 	if (!memcmp(&type, "NAME", 4))
 		misctype = AF_MISC_NAME;
@@ -151,13 +171,10 @@ static status ParseMiscellaneous (AFfilehandle file, AF_VirtualFile *fh,
 	file->miscellaneous[file->miscellaneousCount - 1].id = file->miscellaneousCount;
 	file->miscellaneous[file->miscellaneousCount - 1].type = misctype;
 	file->miscellaneous[file->miscellaneousCount - 1].size = size;
-	/*
-		ftell should probably be replaced by ftell64 on systems such
-		as Irix, but until a 64-bit audio file format comes out, I'm not
-		going to worry about it.
-	*/
-	file->miscellaneous[file->miscellaneousCount - 1].offset = af_ftell(fh);
 	file->miscellaneous[file->miscellaneousCount - 1].position = 0;
+	file->miscellaneous[file->miscellaneousCount - 1].buffer = _af_malloc(size);
+	af_fread(file->miscellaneous[file->miscellaneousCount - 1].buffer,
+		size, 1, file->fh);
 
 	return AF_SUCCEED;
 }
@@ -169,44 +186,53 @@ static status ParseMiscellaneous (AFfilehandle file, AF_VirtualFile *fh,
 static status ParseINST (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 	size_t size)
 {
-	u_int8_t	baseNote, detune, lowNote, highNote, lowVelocity, highVelocity;
-	u_int16_t	gain;
+	_Instrument	*instrument;
+	u_int8_t	baseNote;
+	int8_t		detune;
+	u_int8_t	lowNote, highNote, lowVelocity, highVelocity;
+	int16_t		gain;
 
 	u_int16_t	sustainLoopPlayMode, sustainLoopBegin, sustainLoopEnd;
 	u_int16_t	releaseLoopPlayMode, releaseLoopBegin, releaseLoopEnd;
 
 	assert(!memcmp(&type, "INST", 4));
 
-	file->instrumentCount = 1;
-	file->instruments =
-		(struct _Instrument *) _af_calloc(1, sizeof (struct _Instrument));
+	instrument = _af_calloc(1, sizeof (_Instrument));
+	instrument->id = AF_DEFAULT_INST;
+	instrument->values = _af_calloc(_AF_AIFF_NUM_INSTPARAMS, sizeof (AFPVu));
+	instrument->loopCount = 2;
+	instrument->loops = _af_calloc(2, sizeof (_Loop));
 
-	af_fread(&baseNote, sizeof (u_int8_t), 1, fh);
-	af_fread(&detune, sizeof (u_int8_t), 1, fh);
-	af_fread(&lowNote, sizeof (u_int8_t), 1, fh);
-	af_fread(&highNote, sizeof (u_int8_t), 1, fh);
-	af_fread(&lowVelocity, sizeof (u_int8_t), 1, fh);
-	af_fread(&highVelocity, sizeof (u_int8_t), 1, fh);
-	af_fread(&gain, sizeof (u_int16_t), 1, fh);
+	file->instrumentCount = 1;
+	file->instruments = instrument;
+
+	af_fread(&baseNote, 1, 1, fh);
+	af_fread(&detune, 1, 1, fh);
+	af_fread(&lowNote, 1, 1, fh);
+	af_fread(&highNote, 1, 1, fh);
+	af_fread(&lowVelocity, 1, 1, fh);
+	af_fread(&highVelocity, 1, 1, fh);
+	af_fread(&gain, 2, 1, fh);
 	gain = BENDIAN_TO_HOST_INT16(gain);
 
-	file->instruments[0].id = AF_DEFAULT_INST;
-	file->instruments[0].midiBaseNote = baseNote;
-	file->instruments[0].detune = detune;
-	file->instruments[0].midiLowNote = lowNote;
-	file->instruments[0].midiHighNote = highNote;
-	file->instruments[0].midiLowVelocity = lowVelocity;
-	file->instruments[0].midiHighVelocity = highVelocity;
-	file->instruments[0].gain = gain;
-	file->instruments[0].sustainLoopID = 1;
-	file->instruments[0].releaseLoopID = 2;
-
 #ifdef DEBUG
-	printf(" baseNote/detune/lowNote/highNote/lowVelocity/highVelocity/gain:\n"
+	printf("baseNote/detune/lowNote/highNote/lowVelocity/highVelocity/gain:"
 		" %d %d %d %d %d %d %d\n",
-		baseNote, detune, lowNote, highNote, lowVelocity, highVelocity, gain);
+		baseNote, detune, lowNote, highNote, lowVelocity, highVelocity,
+		gain);
 #endif
 
+	instrument->values[0].l = baseNote;
+	instrument->values[1].l = detune;
+	instrument->values[2].l = lowVelocity;
+	instrument->values[3].l = highVelocity;
+	instrument->values[4].l = lowNote;
+	instrument->values[5].l = highNote;
+	instrument->values[6].l = gain;
+
+	instrument->values[7].l = 1;	/* sustain loop id */
+	instrument->values[8].l = 2;	/* release loop id */
+
 	af_fread(&sustainLoopPlayMode, sizeof (u_int16_t), 1, fh);
 	sustainLoopPlayMode = BENDIAN_TO_HOST_INT16(sustainLoopPlayMode);
 	af_fread(&sustainLoopBegin, sizeof (u_int16_t), 1, fh);
@@ -222,25 +248,22 @@ static status ParseINST (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 	releaseLoopEnd = BENDIAN_TO_HOST_INT16(releaseLoopEnd);
 
 #ifdef DEBUG
-	printf("sustain loop: %d %d %d\n", sustainLoopPlayMode,
-		sustainLoopBegin, sustainLoopEnd);
+	printf("sustain loop: mode %d, begin %d, end %d\n",
+		sustainLoopPlayMode, sustainLoopBegin, sustainLoopEnd);
 
-	printf("release loop: %d %d %d\n", releaseLoopPlayMode,
-		releaseLoopBegin, releaseLoopEnd);
+	printf("release loop: mode %d, begin %d, end %d\n",
+		releaseLoopPlayMode, releaseLoopBegin, releaseLoopEnd);
 #endif
 
-	file->loops = _af_calloc(2, sizeof (struct _Loop));
-	file->loopCount = 2;
+	instrument->loops[0].id = 1;
+	instrument->loops[0].mode = sustainLoopPlayMode;
+	instrument->loops[0].beginMarker = sustainLoopBegin;
+	instrument->loops[0].endMarker = sustainLoopEnd;
 
-	file->loops[0].id = 1;
-	file->loops[0].playMode = sustainLoopPlayMode;
-	file->loops[0].beginLoop = sustainLoopBegin;
-	file->loops[0].endLoop = sustainLoopEnd;
-
-	file->loops[1].id = 2;
-	file->loops[1].playMode = releaseLoopPlayMode;
-	file->loops[1].beginLoop = releaseLoopBegin;
-	file->loops[1].endLoop = releaseLoopEnd;
+	instrument->loops[1].id = 2;
+	instrument->loops[1].mode = releaseLoopPlayMode;
+	instrument->loops[1].beginMarker = releaseLoopBegin;
+	instrument->loops[1].endMarker = releaseLoopEnd;
 
 	return AF_SUCCEED;
 }
@@ -251,22 +274,25 @@ static status ParseINST (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 static status ParseMARK (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 	size_t size)
 {
-	int			i;
+	_Track		*track;
+	int		i;
 	u_int16_t	numMarkers;
 
 	assert(!memcmp(&type, "MARK", 4));
 
+	track = _af_filehandle_get_track(file, AF_DEFAULT_TRACK);
+
 	af_fread(&numMarkers, sizeof (u_int16_t), 1, fh);
 	numMarkers = BENDIAN_TO_HOST_INT16(numMarkers);
 
-	file->markerCount = numMarkers;
-	file->markers = _af_calloc(numMarkers, sizeof (struct _Marker));
+	track->markerCount = numMarkers;
+	track->markers = _af_marker_new(numMarkers);
 
 	for (i=0; i<numMarkers; i++)
 	{
 		u_int16_t	markerID = 0;
 		u_int32_t	markerPosition = 0;
-		unsigned char	sizeByte = 0;
+		u_int8_t	sizeByte = 0;
 		char		*markerName = NULL;
 
 		af_fread(&markerID, sizeof (u_int16_t), 1, fh);
@@ -280,18 +306,25 @@ static status ParseMARK (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 		markerName[sizeByte] = '\0';
 
 #ifdef DEBUG
-		printf("marker id: %d, position: %d, name: %s\n", markerID,
-			markerPosition, markerName);
+		printf("marker id: %d, position: %d, name: %s\n",
+			markerID, markerPosition, markerName);
 
 		printf("size byte: %d\n", sizeByte);
 #endif
 
+		/*
+			If sizeByte is even, then 1+sizeByte (the length
+			of the string) is odd.	Skip an extra byte to
+			make it even.
+		*/
+
 		if ((sizeByte % 2) == 0)
 			af_fseek(fh, 1, SEEK_CUR);
 
-		file->markers[i].id = markerID;
-		file->markers[i].position = markerPosition;
-		file->markers[i].name = markerName;
+		track->markers[i].id = markerID;
+		track->markers[i].position = markerPosition;
+		track->markers[i].name = markerName;
+		track->markers[i].comment = _af_strdup("");
 	}
 
 	return AF_SUCCEED;
@@ -305,24 +338,69 @@ static status ParseMARK (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 static status ParseCOMM (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 	size_t size)
 {
-	u_int16_t		numChannels;
-	u_int32_t		numSampleFrames;
-	u_int16_t		sampleSize;
+	_Track		*track;
+	u_int16_t	numChannels;
+	u_int32_t	numSampleFrames;
+	u_int16_t	sampleSize;
 	unsigned char	sampleRate[10];
 
 	assert(!memcmp(&type, "COMM", 4));
 
+	track = _af_filehandle_get_track(file, AF_DEFAULT_TRACK);
+
 	af_fread(&numChannels, sizeof (u_int16_t), 1, fh);
-	file->channelCount = BENDIAN_TO_HOST_INT16(numChannels);
+	track->f.channelCount = BENDIAN_TO_HOST_INT16(numChannels);
 
 	af_fread(&numSampleFrames, sizeof (u_int32_t), 1, fh);
-	file->frameCount = BENDIAN_TO_HOST_INT32(numSampleFrames);
+	track->totalfframes = BENDIAN_TO_HOST_INT32(numSampleFrames);
 
 	af_fread(&sampleSize, sizeof (u_int16_t), 1, fh);
-	file->sampleWidth = BENDIAN_TO_HOST_INT16(sampleSize);
+	track->f.sampleWidth = BENDIAN_TO_HOST_INT16(sampleSize);
 
 	af_fread(sampleRate, 10, 1, fh);
-	file->sampleRate = ConvertFromIeeeExtended(sampleRate);
+	track->f.sampleRate = ConvertFromIeeeExtended(sampleRate);
+
+	track->f.compressionType = AF_COMPRESSION_NONE;
+	track->f.sampleFormat = AF_SAMPFMT_TWOSCOMP;
+	track->f.byteOrder = AF_BYTEORDER_BIGENDIAN;
+
+	if (file->fileFormat == AF_FILE_AIFFC)
+	{
+		u_int8_t	compressionID[4];
+		/* Pascal strings are at most 255 bytes long. */
+		unsigned char	compressionName[256];
+		unsigned char	compressionNameLength;
+
+		af_fread(compressionID, 4, 1, fh);
+
+		/* Read the Pascal-style string containing the name. */
+		af_fread(&compressionNameLength, 1, 1, fh);
+		af_fread(compressionName, compressionNameLength, 1, fh);
+		compressionName[compressionNameLength] = '\0';
+
+		if (!memcmp(compressionID, "NONE", 4))
+			track->f.compressionType = AF_COMPRESSION_NONE;
+		else if (!memcmp(compressionID, "ACE2", 4) ||
+			!memcmp(compressionID, "ACE8", 4) ||
+			!memcmp(compressionID, "MAC3", 4) ||
+			!memcmp(compressionID, "MAC6", 4))
+		{
+			_af_error(AF_BAD_NOT_IMPLEMENTED, "AIFF-C format does not support Apple's proprietary %s compression format", compressionName);
+			return AF_FAIL;
+		}
+		else if (!memcmp(compressionID, "ulaw", 4) ||
+			!memcmp(compressionID, "ULAW", 4))
+		{
+			track->f.compressionType = AF_COMPRESSION_G711_ULAW;
+		}
+		else if (!memcmp(compressionID, "alaw", 4) ||
+			!memcmp(compressionID, "ALAW", 4))
+		{
+			track->f.compressionType = AF_COMPRESSION_G711_ALAW;
+		}
+	}
+
+	_af_set_sample_format(&track->f, track->f.sampleFormat, track->f.sampleWidth);
 
 	return AF_SUCCEED;
 }
@@ -334,10 +412,13 @@ static status ParseCOMM (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 static status ParseSSND (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 	size_t size)
 {
+	_Track		*track;
 	u_int32_t	offset, blockSize;
 
 	assert(!memcmp(&type, "SSND", 4));
 
+	track = _af_filehandle_get_track(file, AF_DEFAULT_TRACK);
+
 	af_fread(&offset, sizeof (u_int32_t), 1, fh);
 	offset = BENDIAN_TO_HOST_INT32(offset);
 	af_fread(&blockSize, sizeof (u_int32_t), 1, fh);
@@ -347,17 +428,17 @@ static status ParseSSND (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 		This seems like a reasonable way to calculate the number of
 		bytes in an SSND chunk.
 	*/
-	file->trackBytes = size - 8 - offset;
+	track->data_size = size - 8 - offset;
 
 #ifdef DEBUG
 	printf("offset: %d\n", offset);
 	printf("block size: %d\n", blockSize);
 #endif
 
-	file->dataStart = af_ftell(fh) + offset;
+	track->fpos_first_frame = af_ftell(fh) + offset;
 
 #ifdef DEBUG
-	printf("data start: %d\n", file->dataStart);
+	printf("data start: %d\n", track->fpos_first_frame);
 #endif
 
 	/* Sound data follows. */
@@ -371,6 +452,7 @@ status _af_aiff_read_init (AFfilesetup setup, AFfilehandle file)
 	size_t		index = 0;
 	bool		hasCOMM, hasFVER, hasSSND, hasMARK, hasINST;
 	bool		hasAESD, hasNAME, hasAUTH, hasCOPY;
+	_Track		*track;
 
 	hasCOMM = AF_FALSE;
 	hasFVER = AF_FALSE;
@@ -392,26 +474,23 @@ status _af_aiff_read_init (AFfilesetup setup, AFfilehandle file)
 	size = BENDIAN_TO_HOST_INT32(size);
 	af_fread(&formtype, 4, 1, file->fh);
 
-	assert(!memcmp(&type, "FORM", 4));
-	assert(!memcmp(&formtype, "AIFF", 4) || !memcmp(&formtype, "AIFC", 4));
-
-	/*
-		I'm not sure if this is the proper error to throw here. It
-		shouldn't ever be true because of the way file types are
-		distinguished in audiofile.c, but it might be good to have this
-		code anyway.
-	*/
-
-	if (memcmp(&type, "FORM", 4) ||
+	if (memcmp(&type, "FORM", 4) != 0 ||
 		(memcmp(&formtype, "AIFF", 4) && memcmp(&formtype, "AIFC", 4)))
-		_af_error(AF_BAD_AIFF_HEADER, "bad AIFF header");
-	
+		return AF_FAIL;
+
 #ifdef DEBUG
 	printf("size: %d\n", size);
 #endif
 
-	file->sampleFormat = AF_SAMPFMT_TWOSCOMP;
-	file->byteOrder = AF_BYTEORDER_BIGENDIAN;
+	file->instrumentCount = 0;
+	file->instruments = NULL;
+	file->miscellaneousCount = 0;
+	file->miscellaneous = NULL;
+
+	/* AIFF files have only one track. */
+	track = _af_track_new();
+	file->trackCount = 1;
+	file->tracks = track;
 
 	/* Include the offset of the form type. */
 	index += 4;
@@ -424,7 +503,6 @@ status _af_aiff_read_init (AFfilesetup setup, AFfilehandle file)
 		printf("index: %d\n", index);
 #endif
 		af_fread(&chunkid, 4, 1, file->fh);
-/*		chunkid = BENDIAN_TO_HOST_INT32(chunkid); */
 		af_fread(&chunksize, 4, 1, file->fh);
 		chunksize = BENDIAN_TO_HOST_INT32(chunksize);
 
@@ -473,6 +551,11 @@ status _af_aiff_read_init (AFfilesetup setup, AFfilehandle file)
 		*/
 		else if (!memcmp("SSND", &chunkid, 4))
 		{
+			if (hasSSND)
+			{
+				_af_error(AF_BAD_AIFF_SSND, "AIFF file has more than one SSND chunk");
+				return AF_FAIL;
+			}
 			hasSSND = AF_TRUE;
 			ParseSSND(file, file->fh, chunkid, chunksize);
 		}
@@ -491,8 +574,8 @@ status _af_aiff_read_init (AFfilesetup setup, AFfilehandle file)
 		_af_error(AF_BAD_AIFF_COMM, "bad AIFF COMM chunk");
 	}
 
-	/* A zero return value indicates successful parsing. */
-	return 0;
+	/* The file has been successfully parsed. */
+	return AF_SUCCEED;
 }
 
 bool _af_aiff_recognize (AFvirtualfile *fh)
@@ -525,7 +608,102 @@ bool _af_aifc_recognize (AFvirtualfile *fh)
 
 AFfilesetup _af_aiff_complete_setup (AFfilesetup setup)
 {
-	return AF_NULL_FILESETUP;
+	_TrackSetup	*track;
+
+	bool	isAIFF = setup->fileFormat == AF_FILE_AIFF;
+
+	if (setup->trackSet && setup->trackCount != 1)
+	{
+		_af_error(AF_BAD_NUMTRACKS, "AIFF/AIFF-C file must have 1 track");
+		return AF_NULL_FILESETUP;
+	}
+
+	track = &setup->tracks[0];
+
+	if (track->sampleFormatSet)
+	{
+		if (track->f.sampleFormat == AF_SAMPFMT_UNSIGNED)
+		{
+			_af_error(AF_BAD_FILEFMT, "AIFF/AIFF-C format does not support unsigned data");
+			return AF_NULL_FILESETUP;
+		}
+		else if (isAIFF && track->f.sampleFormat != AF_SAMPFMT_TWOSCOMP)
+		{
+			_af_error(AF_BAD_FILEFMT, "AIFF format supports only two's complement integer data");
+			return AF_NULL_FILESETUP;
+		}
+	}
+	else
+		_af_set_sample_format(&track->f, AF_SAMPFMT_TWOSCOMP,
+			track->f.sampleWidth);
+
+	/* Check sample width if writing two's complement. Otherwise ignore. */
+	if (track->f.sampleFormat == AF_SAMPFMT_TWOSCOMP &&
+		(track->f.sampleWidth < 1 || track->f.sampleWidth > 32))
+	{
+		_af_error(AF_BAD_WIDTH,
+			"invalid sample width %d for AIFF/AIFF-C file "
+			"(must be 1-32)", track->f.sampleWidth);
+		return AF_NULL_FILESETUP;
+	}
+
+	if (isAIFF && track->f.compressionType != AF_COMPRESSION_NONE)
+	{
+		_af_error(AF_BAD_FILESETUP,
+			"AIFF does not support compression; use AIFF-C");
+		return AF_NULL_FILESETUP;
+	}
+
+	/* XXXmpruett handle compression here */
+
+	if (track->byteOrderSet &&
+		track->f.byteOrder != AF_BYTEORDER_BIGENDIAN &&
+		track->f.sampleWidth > 8)
+	{
+		_af_error(AF_BAD_BYTEORDER,
+			"AIFF/AIFF-C format supports only big-endian data");
+	}
+	track->f.byteOrder = AF_BYTEORDER_BIGENDIAN;
+
+	if (setup->instrumentSet)
+	{
+		if (setup->instrumentCount != 0 && setup->instrumentCount != 1)
+		{
+			_af_error(AF_BAD_NUMINSTS, "AIFF/AIFF-C file must have 0 or 1 instrument chunk");
+			return AF_NULL_FILESETUP;
+		}
+		if (setup->instruments != 0 &&
+			setup->instruments[0].loopCount != 2)
+		{
+			_af_error(AF_BAD_NUMLOOPS, "AIFF/AIFF-C file with instrument must also have 2 loops");
+			return AF_NULL_FILESETUP;
+		}
+	}
+
+	if (setup->miscellaneousSet)
+	{
+		int	i;
+		for (i=0; i<setup->miscellaneousCount; i++)
+		{
+			switch (setup->miscellaneous[i].type)
+			{
+				case AF_MISC_COPY:
+				case AF_MISC_AUTH:
+				case AF_MISC_NAME:
+				case AF_MISC_ANNO:
+				case AF_MISC_APPL:
+				case AF_MISC_MIDI:
+					break;
+
+				default:
+					_af_error(AF_BAD_MISCTYPE, "invalid miscellaneous type %d for AIFF/AIFF-C file", setup->miscellaneous[i].type);
+					return AF_NULL_FILESETUP;
+					break;
+			}
+		}
+	}
+
+	return _af_filesetup_copy(setup, &_af_aiff_default_filesetup, AF_TRUE);
 }
 
 bool _af_aiff_instparam_valid (AFfilehandle filehandle, AUpvlist list, int i)
