@@ -1,6 +1,7 @@
 /*
 	Audio File Library
 	Copyright (C) 1998-2000, Michael Pruett <michael@68k.org>
+	Copyright (C) 2000, Silicon Graphics, Inc.
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Library General Public
@@ -13,8 +14,8 @@
 	Library General Public License for more details.
 
 	You should have received a copy of the GNU Library General Public
-	License along with this library; if not, write to the 
-	Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
+	License along with this library; if not, write to the
+	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 	Boston, MA  02111-1307  USA.
 */
 
@@ -34,21 +35,35 @@
 #include "audiofile.h"
 #include "aiff.h"
 #include "byteorder.h"
-#include "block.h"
+#include "util.h"
+#include "setup.h"
 
-int aiffWriteFrames (const AFfilehandle file, int track, void *samples, const int count);
 status _af_aiff_update (AFfilehandle file);
-status aiffOpenFileWrite (AFfilehandle file);
 
-static status WriteCOMM (const AFfilehandle file);
-static AFframecount WriteSSND (const AFfilehandle file, void *samples, size_t count);
+static status WriteCOMM (AFfilehandle file);
+static status WriteSSND (AFfilehandle file);
 static status WriteMARK (AFfilehandle file);
 static status WriteINST (AFfilehandle file);
 static status WriteFVER (AFfilehandle file);
 static status WriteAESD (AFfilehandle file);
 static status WriteMiscellaneous (AFfilehandle file);
 
-status aiffOpenFileWrite (AFfilehandle file)
+static _AIFFInfo *aiffinfo_new ()
+{
+	_AIFFInfo	*aiff = _af_malloc(sizeof (_AIFFInfo));
+
+	aiff->miscellaneousPosition = 0;
+	aiff->FVER_offset = 0;
+	aiff->COMM_offset = 0;
+	aiff->MARK_offset = 0;
+	aiff->INST_offset = 0;
+	aiff->AESD_offset = 0;
+	aiff->SSND_offset = 0;
+
+	return aiff;
+}
+
+status _af_aiff_write_init (AFfilesetup setup, AFfilehandle file)
 {
 	u_int32_t	fileSize = HOST_TO_BENDIAN_INT32(0);
 
@@ -56,6 +71,11 @@ status aiffOpenFileWrite (AFfilehandle file)
 	assert(file->fileFormat == AF_FILE_AIFF ||
 		file->fileFormat == AF_FILE_AIFFC);
 
+	if (_af_filesetup_make_handle(setup, file) == AF_FAIL)
+		return AF_FAIL;
+
+	file->formatSpecific = aiffinfo_new();
+
 	af_fwrite("FORM", 4, 1, file->fh);
 	af_fwrite(&fileSize, 4, 1, file->fh);
 
@@ -68,79 +88,67 @@ status aiffOpenFileWrite (AFfilehandle file)
 		WriteFVER(file);
 
 	WriteCOMM(file);
-	WriteAESD(file);
 	WriteMARK(file);
 	WriteINST(file);
+	WriteAESD(file);
 	WriteMiscellaneous(file);
+	WriteSSND(file);
 
 	return AF_SUCCEED;
 }
 
-/* aiffWriteFrames is endian-clean. */
-int aiffWriteFrames (const AFfilehandle file, int track, void *samples, const int count)
-{
-	AFframecount	finalCount;
-
-	assert(file);
-	assert(samples);
-
-	finalCount = WriteSSND(file, samples, count);
-
-#ifdef DEBUG
-	printf("aiffWriteFrames\n");
-	_af_printfilehandle(file);
-#endif
-
-	return finalCount;
-}
-
 status _af_aiff_update (AFfilehandle file)
 {
+	_Track		*track;
 	u_int32_t	length;
 
 	assert(file);
 
+	track = _af_filehandle_get_track(file, AF_DEFAULT_TRACK);
+
 #ifdef DEBUG
 	printf("_af_aiff_update called.\n");
 #endif
 
-	if (file->dataStart != 0)
-	{
-		/* Get the length of the file. */
-		length = af_flength(file->fh);
-		length -= 8;
-		length = HOST_TO_BENDIAN_INT32(length);
+	/* Get the length of the file. */
+	length = af_flength(file->fh);
+	length -= 8;
+	length = HOST_TO_BENDIAN_INT32(length);
 
-		/* Set the length of the FORM chunk. */
-		af_fseek(file->fh, 4, SEEK_SET);
-		af_fwrite(&length, 4, 1, file->fh);
+	/* Set the length of the FORM chunk. */
+	af_fseek(file->fh, 4, SEEK_SET);
+	af_fwrite(&length, 4, 1, file->fh);
 
-		/* Update the SSND chunk's chunk size. */
-		af_fseek(file->fh, file->dataStart - 12, SEEK_SET);
-		length =
-			file->frameCount * ((file->sampleWidth + 7) / 8) * file->channelCount + 8;
-		length = HOST_TO_BENDIAN_INT32(length);
-		af_fwrite(&length, 4, 1, file->fh);
+	if (file->fileFormat == AF_FILE_AIFFC)
+		WriteFVER(file);
 
-		/* Update the COMM chunk's count of sample frames. */
-		af_fseek(file->fh, 12, SEEK_SET);
-		if (file->fileFormat == AF_FILE_AIFFC)
-			WriteFVER(file);
-
-		WriteCOMM(file);
-		WriteAESD(file);
-	}
+	WriteCOMM(file);
+	WriteMARK(file);
+	WriteINST(file);
+	WriteAESD(file);
+	WriteMiscellaneous(file);
+	WriteSSND(file);
 
 	return AF_SUCCEED;
 }
 
-/* WriteCOMM is believed to be endian-clean. */
 static status WriteCOMM (const AFfilehandle file)
 {
-	u_int32_t		size;
-	u_int16_t		sb;
-	u_int32_t		lb;
+	_Track		*track;
+	u_int32_t	size;
+	u_int16_t	sb;
+	u_int32_t	lb;
 	unsigned char	eb[10];
+	_AIFFInfo	*aiff;
+
+	aiff = file->formatSpecific;
+
+	if (aiff->COMM_offset == 0)
+		aiff->COMM_offset = af_ftell(file->fh);
+	else
+		af_fseek(file->fh, aiff->COMM_offset, SEEK_SET);
+
+	track = _af_filehandle_get_track(file, AF_DEFAULT_TRACK);
 
 	size = 18;
 	if (file->fileFormat == AF_FILE_AIFFC)
@@ -151,19 +159,19 @@ static status WriteCOMM (const AFfilehandle file)
 	af_fwrite(&size, 4, 1, file->fh);
 
 	/* number of channels, 2 bytes */
-	sb = HOST_TO_BENDIAN_INT16(file->channelCount);
+	sb = HOST_TO_BENDIAN_INT16(track->f.channelCount);
 	af_fwrite(&sb, 2, 1, file->fh);
 
 	/* number of sample frames, 4 bytes */
-	lb = HOST_TO_BENDIAN_INT32(file->frameCount);
+	lb = HOST_TO_BENDIAN_INT32(track->totalfframes);
 	af_fwrite(&lb, 4, 1, file->fh);
 
 	/* sample size, 2 bytes */
-	sb = HOST_TO_BENDIAN_INT16(file->sampleWidth);
+	sb = HOST_TO_BENDIAN_INT16(track->f.sampleWidth);
 	af_fwrite(&sb, 2, 1, file->fh);
 
 	/* sample rate, 10 bytes */
-	ConvertToIeeeExtended(file->sampleRate, eb);
+	ConvertToIeeeExtended(track->f.sampleRate, eb);
 	af_fwrite(eb, 10, 1, file->fh);
 
 	if (file->fileFormat == AF_FILE_AIFFC)
@@ -184,79 +192,83 @@ static status WriteCOMM (const AFfilehandle file)
 	return AF_SUCCEED;
 }
 
-/* The AESD chunk contains information pertinent to audio recording devices. */
-
 /*
-	In files created by the Audio File Library, the AESD chunk always
-	follows the COMM chunk for our convenience.  Whenever the file is
-	created or the file is synchronized, these two chunks are always
-	rewritten as AES data is able to be changed after afOpenFile.
+	The AESD chunk contains information pertinent to audio recording
+	devices.
 */
 static status WriteAESD (const AFfilehandle file)
 {
+	_Track		*track;
 	u_int32_t	size = 24;
+	_AIFFInfo	*aiff;
 
 	assert(file);
 
-	if (!file->aesDataPresent)
-		return;
+	aiff = file->formatSpecific;
 
-	af_fwrite("AESD", 4, 1, file->fh);
+	track = _af_filehandle_get_track(file, AF_DEFAULT_TRACK);
+
+	if (track->hasAESData == AF_FALSE)
+		return AF_SUCCEED;
+
+	if (aiff->AESD_offset == 0)
+		aiff->AESD_offset = af_ftell(file->fh);
+	else
+		af_fseek(file->fh, aiff->AESD_offset, SEEK_SET);
+
+	if (af_fwrite("AESD", 4, 1, file->fh) < 1)
+		return AF_FAIL;
 
 	size = HOST_TO_BENDIAN_INT32(size);
-	af_fwrite(&size, 4, 1, file->fh);
 
-	af_fwrite(file->aesData, 24, 1, file->fh);
+	if (af_fwrite(&size, 4, 1, file->fh) < 1)
+		return AF_FAIL;
+
+	if (af_fwrite(track->aesData, 24, 1, file->fh) < 1)
+		return AF_FAIL;
 
 	return AF_SUCCEED;
 }
 
-/* WriteSSND is believed to be endian-clean. */
-static AFframecount WriteSSND (const AFfilehandle file, void *samples, size_t count)
+static status WriteSSND (AFfilehandle file)
 {
-	u_int32_t	length, chunkSize, frameSize, zero = 0;
-	AFframecount	finalCount = 0;
+	_Track		*track;
+	u_int32_t	chunkSize, zero = 0;
+	_AIFFInfo	*aiff;
 
 	assert(file);
 	assert(file->fh);
-	assert(samples);
 
-#ifdef DEBUG
-	printf("WriteSSND called.\n");
-#endif
+	aiff = file->formatSpecific;
 
-	frameSize = file->channelCount * ((file->sampleWidth + 7) / 8);
+	track = _af_filehandle_get_track(file, AF_DEFAULT_TRACK);
 
-	length = (file->frameCount + count) * frameSize;
-	chunkSize = length + 8;
+	if (aiff->SSND_offset == 0)
+		aiff->SSND_offset = af_ftell(file->fh);
+	else
+		af_fseek(file->fh, aiff->SSND_offset, SEEK_SET);
 
-#ifdef DEBUG
-	printf("count: %d\n", count);
-	printf("chunkSize: %d\n", chunkSize);
-#endif
+	chunkSize = _af_format_frame_size(&track->f, AF_FALSE) *
+		track->totalfframes + 8;
 
-	if (file->dataStart == 0)
-	{
-		af_fwrite("SSND", 4, 1, file->fh);
-		/* initial size set to zero */
-		af_fwrite(&zero, 4, 1, file->fh);
+	af_fwrite("SSND", 4, 1, file->fh);
+	chunkSize = HOST_TO_BENDIAN_INT32(chunkSize);
+	af_fwrite(&chunkSize, 4, 1, file->fh);
 
-		/* offset */
-		af_fwrite(&zero, 4, 1, file->fh);
-		/* block size */
-		af_fwrite(&zero, 4, 1, file->fh);
+	/* data offset */
+	af_fwrite(&zero, 4, 1, file->fh);
+	/* block size */
+	af_fwrite(&zero, 4, 1, file->fh);
 
-		file->dataStart = af_ftell(file->fh);
-	}
+	if (track->fpos_first_frame == 0)
+		track->fpos_first_frame = af_ftell(file->fh);
 
-	finalCount = _af_blockWriteFrames(file, AF_DEFAULT_TRACK, samples, count);
-
-	return finalCount;
+	return AF_SUCCEED;
 }
 
 static status WriteINST (AFfilehandle file)
 {
-	u_int32_t		length;
+	u_int32_t	length;
 	struct _INST	instrumentdata;
 
 	length = 20;
@@ -314,25 +326,33 @@ static status WriteINST (AFfilehandle file)
 	return AF_SUCCEED;
 }
 
-/* This function is probably fairly endian-clean. */
 static status WriteMARK (AFfilehandle file)
 {
-	off_t		start, end;
-	u_int32_t	length;
+	off_t		chunkStartPosition, chunkEndPosition;
+	u_int32_t	length = 0;
 	u_int16_t	numMarkers, sb;
-	int			i, *markids;
+	int		i, *markids;
+	_Track		*track;
+	_AIFFInfo	*aiff;
 
 	assert(file);
 
-	length = 0;
+	aiff = file->formatSpecific;
+
+	track = _af_filehandle_get_track(file, AF_DEFAULT_TRACK);
+
+	if (aiff->MARK_offset == 0)
+		aiff->MARK_offset = af_ftell(file->fh);
+	else
+		af_fseek(file->fh, aiff->MARK_offset, SEEK_SET);
 
 	af_fwrite("MARK", 4, 1, file->fh);
 	af_fwrite(&length, 4, 1, file->fh);
 
-	start = af_ftell(file->fh);
+	chunkStartPosition = af_ftell(file->fh);
 
-	numMarkers = file->markerCount;
-	markids = (int *) malloc(numMarkers * sizeof (int));
+	numMarkers = track->markerCount;
+	markids = _af_malloc(numMarkers * sizeof (int));
 	afGetMarkIDs(file, AF_DEFAULT_TRACK, markids);
 
 	sb = HOST_TO_BENDIAN_INT16(numMarkers);
@@ -354,36 +374,48 @@ static status WriteMARK (AFfilehandle file)
 		af_fwrite(&position, 4, 1, file->fh);
 
 		/* Write the strings in Pascal style. */
-		assert(file->markers[i].name);
-		namelength = strlen(file->markers[i].name);
+		assert(track->markers[i].name);
+		namelength = strlen(track->markers[i].name);
 		af_fwrite(&namelength, 1, 1, file->fh);
-		af_fwrite(file->markers[i].name, 1, namelength, file->fh);
+		af_fwrite(track->markers[i].name, 1, namelength, file->fh);
 
 		if ((namelength % 2) == 0)
 			af_fwrite(&zero, 1, 1, file->fh);
 	}
 
-	end = af_ftell(file->fh);
-	length = end - start;
+	chunkEndPosition = af_ftell(file->fh);
+	length = chunkEndPosition - chunkStartPosition;
 
 #ifdef DEBUG
-	printf(" end: %d\n", end);
-	printf(" length: %d\n", end - start);
+	printf(" end: %d\n", chunkEndPosition);
+	printf(" length: %d\n", length);
 #endif
 
-	af_fseek(file->fh, start - 4, SEEK_SET);
+	af_fseek(file->fh, chunkStartPosition - 4, SEEK_SET);
 
 	length = HOST_TO_BENDIAN_INT32(length);
 	af_fwrite(&length, 4, 1, file->fh);
-	af_fseek(file->fh, end, SEEK_SET);
+	af_fseek(file->fh, chunkEndPosition, SEEK_SET);
 
 	return AF_SUCCEED;
 }
 
-/* This function is endian-clean. */
+/*
+	The FVER chunk, if present, is always the first chunk in the file.
+*/
 static status WriteFVER (AFfilehandle file)
 {
 	u_int32_t	chunkSize, timeStamp;
+	_AIFFInfo	*aiff;
+
+	assert(file->fileFormat == AF_FILE_AIFFC);
+
+	aiff = file->formatSpecific;
+
+	if (aiff->FVER_offset == 0)
+		aiff->FVER_offset = af_ftell(file->fh);
+	else
+		af_fseek(file->fh, aiff->FVER_offset, SEEK_SET);
 
 	af_fwrite("FVER", 4, 1, file->fh);
 
@@ -404,12 +436,21 @@ static status WriteFVER (AFfilehandle file)
 */
 static status WriteMiscellaneous (AFfilehandle file)
 {
-	int	i;
+	_AIFFInfo	*aiff;
+	int		i;
+
+	aiff = (_AIFFInfo *) file->formatSpecific;
+
+	if (aiff->miscellaneousPosition == 0)
+		aiff->miscellaneousPosition = af_ftell(file->fh);
+	else
+		af_fseek(file->fh, aiff->miscellaneousPosition, SEEK_SET);
 
 	for (i=0; i<file->miscellaneousCount; i++)
 	{
-		struct _Miscellaneous	*misc = &file->miscellaneous[i];
-		u_int32_t				chunkType, chunkSize;
+		_Miscellaneous	*misc = &file->miscellaneous[i];
+		u_int32_t	chunkType, chunkSize;
+		u_int8_t	padByte = 0;
 
 #ifdef DEBUG
 		printf("WriteMiscellaneous: %d, type %d\n", i, misc->type);
@@ -435,18 +476,19 @@ static status WriteMiscellaneous (AFfilehandle file)
 
 		af_fwrite(&chunkType, 4, 1, file->fh);
 		af_fwrite(&chunkSize, 4, 1, file->fh);
-		misc->offset = af_ftell(file->fh);
 		/*
-			Skip the appropriate number of bytes, leaving room for a
-			pad byte.
+			Write the miscellaneous buffer and then a pad byte
+			if necessary.  If the buffer is null, skip the space
+			for now.
 		*/
-		af_fseek(file->fh, misc->size + (misc->size % 2), SEEK_CUR);
+		if (misc->buffer != NULL)
+			af_fwrite(misc->buffer, misc->size, 1, file->fh);
+		else
+			af_fseek(file->fh, misc->size, SEEK_CUR);
+
+		if (misc->size % 2 != 0)
+			af_fwrite(&padByte, 1, 1, file->fh);
 	}
 
 	return AF_SUCCEED;
 }
-
-status _af_aiff_write_init (AFfilesetup setup, AFfilehandle filehandle)
-{
-	return AF_FAIL;
-}
