@@ -13,8 +13,8 @@
 	Library General Public License for more details.
 
 	You should have received a copy of the GNU Library General Public
-	License along with this library; if not, write to the 
-	Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
+	License along with this library; if not, write to the
+	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 	Boston, MA  02111-1307  USA.
 */
 
@@ -35,7 +35,7 @@
 #include "aupvlist.h"
 #include "error.h"
 #include "util.h"
-#include "unit.h"
+#include "units.h"
 #include "instrument.h"
 
 extern _Unit _af_units[];
@@ -321,7 +321,7 @@ AUpvlist _afQueryInstrumentParameter (int arg1, int arg2, int arg3, int arg4)
 			if (arg2 < 0 || arg2 >= _AF_NUM_UNITS)
 				return AU_NULL_PVLIST;
 
-			idx = _af_instparam_index_from_instparam_id(arg2, arg3);
+			idx = _af_instparam_index_from_id(arg2, arg3);
 			if (idx<0)
 				return AU_NULL_PVLIST;
 			return _af_pv_long(_af_units[arg2].instrumentParameters[idx].type);
@@ -333,7 +333,7 @@ AUpvlist _afQueryInstrumentParameter (int arg1, int arg2, int arg3, int arg4)
 
 			if (arg2 < 0 || arg2 >= _AF_NUM_UNITS)
 				return AU_NULL_PVLIST;
-			idx = _af_instparam_index_from_instparam_id(arg2, arg3);
+			idx = _af_instparam_index_from_id(arg2, arg3);
 			if (idx < 0)
 				return AU_NULL_PVLIST;
 			return _af_pv_pointer(_af_units[arg2].instrumentParameters[idx].name);
@@ -345,7 +345,7 @@ AUpvlist _afQueryInstrumentParameter (int arg1, int arg2, int arg3, int arg4)
 
 			if (arg2 < 0 || arg2 >= _AF_NUM_UNITS)
 				return AU_NULL_PVLIST;
-			idx = _af_instparam_index_from_instparam_id(arg2, arg3);
+			idx = _af_instparam_index_from_id(arg2, arg3);
 			if (idx >= 0)
 			{
 				AUpvlist	ret = AUpvnew(1);
