@@ -13,8 +13,8 @@
 	Library General Public License for more details.
 
 	You should have received a copy of the GNU Library General Public
-	License along with this library; if not, write to the 
-	Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
+	License along with this library; if not, write to the
+	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 	Boston, MA  02111-1307  USA.
 */
 
@@ -28,11 +28,9 @@
 #include <stddef.h>
 #include <assert.h>
 #include "audiofile.h"
+#include "afinternal.h"
+#include "util.h"
 
-/*
-	afInitTrackIDs is implemented in just about the same manner as the
-	SGI version: barely.
-*/
 void afInitTrackIDs (AFfilesetup file, int *trackids, int trackCount)
 {
 	assert(file);
@@ -41,10 +39,6 @@ void afInitTrackIDs (AFfilesetup file, int *trackids, int trackCount)
 	assert(trackids[0] == AF_DEFAULT_TRACK);
 }
 
-/*
-	afGetTrackIDs is implemented in just about the same manner as the
-	SGI version: barely.
-*/
 int afGetTrackIDs (AFfilehandle file, int *trackids)
 {
 	assert(file);
@@ -54,3 +48,40 @@ int afGetTrackIDs (AFfilehandle file, int *trackids)
 
 	return 1;
 }
+
+_Track *_af_track_new (void)
+{
+	_Track	*t = _af_malloc(sizeof (_Track));
+
+	t->valid = _AF_VALID_TRACK;
+	t->id = AF_DEFAULT_TRACK;
+
+	t->f.compressionParams = NULL;
+	t->v.compressionParams = NULL;
+
+	t->channelMatrix = NULL;
+
+	t->markerCount = 0;
+	t->markers = NULL;
+
+	t->hasAESData = AF_FALSE;
+	memset(t->aesData, 0, 24);
+
+	t->totalfframes = 0;
+	t->nextfframe = 0;
+	t->frames2ignore = 0;
+	t->fpos_first_frame = 0;
+	t->fpos_next_frame = 0;
+	t->fpos_after_data = 0;
+	t->totalvframes = 0;
+	t->nextvframe = 0;
+	t->data_size = 0;
+
+	t->ms.modulesdirty = AF_TRUE;
+	t->ms.nmodules = 0;
+	t->ms.chunk = NULL;
+	t->ms.module = NULL;
+	t->ms.buffer = NULL;
+
+	return t;
+}
